@@ -17,7 +17,6 @@ import (
 	"strings"
 
 	"github.com/blacktop/go-macho"
-	"github.com/blacktop/go-plist"
 	fwcmd "github.com/blacktop/ipsw/internal/commands/fw"
 	"github.com/blacktop/ipsw/internal/download"
 	"github.com/blacktop/ipsw/internal/utils"
@@ -28,6 +27,7 @@ import (
 	"github.com/blacktop/ipsw/pkg/kernelcache"
 	"github.com/blacktop/ipsw/pkg/lzfse"
 	"github.com/blacktop/ipsw/pkg/ota"
+	"github.com/blacktop/ipsw/pkg/plist"
 )
 
 // Config is the extract command configuration.
@@ -42,6 +42,8 @@ type Config struct {
 	Arches []string `json:"arches,omitempty"`
 	// extract the DriverKit DSCs
 	DriverKit bool `json:"driver_kit,omitempty"`
+	// extract the DriverKit DSCs
+	AllDSCs bool `json:"all_dscs,omitempty"`
 	// extract a single device's kernelcache
 	KernelDevice string `json:"kernel_device,omitempty"`
 	// http proxy to use
@@ -305,7 +307,7 @@ func DSC(c *Config) ([]string, error) {
 		if err != nil {
 			return nil, err
 		}
-		return dyld.Extract(c.IPSW, filepath.Join(filepath.Clean(c.Output), folder), c.Arches, c.DriverKit)
+		return dyld.Extract(c.IPSW, filepath.Join(filepath.Clean(c.Output), folder), c.Arches, c.DriverKit, c.AllDSCs)
 	} else if len(c.URL) > 0 {
 		if !isURL(c.URL) {
 			return nil, fmt.Errorf("invalid URL provided: %s", c.URL)
@@ -316,7 +318,7 @@ func DSC(c *Config) ([]string, error) {
 		}
 		if i.Plists.Type == "OTA" {
 			if runtime.GOOS == "darwin" {
-				out, err := dyld.ExtractFromRemoteCryptex(zr, filepath.Join(filepath.Clean(c.Output), folder), c.Arches, c.DriverKit)
+				out, err := dyld.ExtractFromRemoteCryptex(zr, filepath.Join(filepath.Clean(c.Output), folder), c.Arches, c.DriverKit, c.AllDSCs)
 				if err != nil {
 					if errors.Is(err, dyld.ErrNoCryptex) {
 						c.Pattern = `^` + dyld.CacheRegex
@@ -366,7 +368,7 @@ func DSC(c *Config) ([]string, error) {
 		if _, err := utils.SearchZip(zr.File, regexp.MustCompile(fmt.Sprintf("^%s$", sysDMG)), tmpDIR, c.Flatten, true); err != nil {
 			return nil, fmt.Errorf("failed to extract SystemOS DMG from remote IPSW: %v", err)
 		}
-		return dyld.ExtractFromDMG(i, filepath.Join(tmpDIR, sysDMG), filepath.Join(filepath.Clean(c.Output), folder), c.Arches, c.DriverKit)
+		return dyld.ExtractFromDMG(i, filepath.Join(tmpDIR, sysDMG), filepath.Join(filepath.Clean(c.Output), folder), c.Arches, c.DriverKit, c.AllDSCs)
 	}
 	return nil, fmt.Errorf("no IPSW or URL provided")
 }
@@ -746,19 +748,8 @@ func LaunchdConfig(path string) (string, error) {
 	return string(data), nil
 }
 
-// SystemVersionPlist is the SystemVersion.plist struct
-type SystemVersionPlist struct {
-	BuildID             string `json:"build_id,omitempty"`
-	ProductBuildVersion string `json:"product_build_version,omitempty"`
-	ProductCopyright    string `json:"product_copyright,omitempty"`
-	ProductName         string `json:"product_name,omitempty"`
-	ProductVersion      string `json:"product_version,omitempty"`
-	ReleaseType         string `json:"release_type,omitempty"`
-	SystemImageID       string `json:"system_image_id,omitempty"`
-}
-
 // SystemVersion extracts the system version info from an IPSW
-func SystemVersion(path string) (*SystemVersionPlist, error) {
+func SystemVersion(path string) (*plist.SystemVersion, error) {
 	ipswPath := filepath.Clean(path)
 
 	i, err := info.Parse(ipswPath)
@@ -787,10 +778,5 @@ func SystemVersion(path string) (*SystemVersionPlist, error) {
 		return nil, fmt.Errorf("failed to read SystemVersion.plist: %v", err)
 	}
 
-	var sysVer SystemVersionPlist
-	if err := plist.NewDecoder(bytes.NewReader(dat)).Decode(&sysVer); err != nil {
-		return nil, fmt.Errorf("failed to decode SystemVersion.plist: %v", err)
-	}
-
-	return &sysVer, nil
+	return plist.ParseSystemVersion(dat)
 }
