@@ -33,6 +33,7 @@ func (b *BuildManifest) String() string {
 }
 
 type buildIdentity struct {
+	ApOSLongVersion               string                      `plist:"Ap,OSLongVersion,omitempty" json:"ap_os_long_version,omitempty"`
 	ApBoardID                     string                      `json:"ap_board_id,omitempty"`
 	ApChipID                      string                      `json:"ap_chip_id,omitempty"`
 	ApSecurityDomain              string                      `json:"ap_security_domain,omitempty"`
@@ -49,7 +50,13 @@ type buildIdentity struct {
 
 func (i buildIdentity) String() string {
 	var out string
-	out += fmt.Sprintf("    ProductMarketingVersion: %s\n", i.ProductMarketingVersion)
+	if len(i.ProductMarketingVersion) > 0 {
+		out += fmt.Sprintf("    ProductMarketingVersion: %s\n", i.ProductMarketingVersion)
+	}
+	if len(i.ApOSLongVersion) > 0 {
+		out += fmt.Sprintf("    Ap,OSLongVersion:        %s\n", i.ApOSLongVersion)
+	}
+	out += fmt.Sprintf("    ApBoardID:               %s\n", i.ApBoardID)
 	out += fmt.Sprintf("    ApBoardID:               %s\n", i.ApBoardID)
 	out += fmt.Sprintf("    ApChipID:                %s\n", i.ApChipID)
 	out += fmt.Sprintf("    ApSecurityDomain:        %s\n", i.ApSecurityDomain)
